#!/bin/sh

# Copyright (C) 2019, 2020 Svante Signell

# Written by Svante Signell  <svante.signell@gmail.com>

# This file is part of hurd-cross.

# Hurd-cross is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 3 or (at your
# option) any later version.

# The hurd-cross is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the GNU Hurd.  If not, see <http://www.gnu.org/licenses/>.

. ./bootstrap-funcs.sh

echo ""
echo "File: bootstrap.sh"

print_info "************************************************************************"
print_info "Bootstrapping binutils, glibc and gcc on $HOST to $TARGET"
print_info "************************************************************************"

cd $SRC

if [ ! -f $STAMPS_BOOTSTRAP/$BINUTILS_SRC ]; then
    print_info "Cross-compiling binutils"
    compile_binutils
    touch $STAMPS_BOOTSTRAP/$BINUTILS_SRC
fi
#exit
if [ ! -f $STAMPS_BOOTSTRAP/$GNUMACH_SRC-headers ]; then
    print_info "Installing GNU Mach headers"
    install_gnumach_headers
    touch $STAMPS_BOOTSTRAP/$GNUMACH_SRC-headers
fi
#exit
if [ ! -f $STAMPS_BOOTSTRAP/$HURD_SRC-headers ]; then
    print_info "Installing Hurd headers"
    install_hurd_headers
    touch $STAMPS_BOOTSTRAP/$HURD_SRC-headers
fi
#exit
#apt-get install libgmp-dev libmpc-dev libmpfr-dev
if [ ! -f $STAMPS_BOOTSTRAP/first-$GCC_SRC ]; then
    compile_first_gcc
    touch $STAMPS_BOOTSTRAP/first-$GCC_SRC
fi
#exit
#FIXED: Needs i686-pc-gnu-gcc
#apt-get install libbison-dev flex libfl-dev
if [ ! -f $STAMPS_BOOTSTRAP/$GNUMIG_SRC-git ]; then
    print_info "Cross-compiling gnumig"
    install_gnumig
    touch $STAMPS_BOOTSTRAP/$GNUMIG_SRC-git
fi
#exit
#apt-get install gawk
if [ ! -f $STAMPS_BOOTSTRAP/first-$GLIBC_SRC ]; then
    compile_first_glibc
    touch $STAMPS_BOOTSTRAP/first-$GLIBC_SRC
fi
#exit
# Fixes a bug in fixincludes: 
# patches/gcc/fixinc_glimits.h.diff applied in download.sh
# FIXME: apply+unapply patches/gcc/mkfixinc.sh.diff
if [ ! -f $STAMPS_BOOTSTRAP/full-$GCC_SRC ]; then
    compile_full_gcc
    touch $STAMPS_BOOTSTRAP/full-$GCC_SRC
fi
#exit
if [ ! -f $STAMPS_BOOTSTRAP/full-$GLIBC_SRC ]; then
    compile_full_glibc
    touch $STAMPS_BOOTSTRAP/full-$GLIBC_SRC
fi

print_info "***********************"
print_info "Bootstrapping completed"
print_info "***********************"

exit 0
