#!/bin/sh

# Copyright (C) 2019, 2020 Svante Signell

# Written by Svante Signell  <svante.signell@gmail.com>

# This file is part of hurd-cross.

# Hurd-cross is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 3 or (at your
# option) any later version.

# The hurd-cross is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the GNU Hurd.  If not, see <http://www.gnu.org/licenses/>.

. ./package-versions.sh
. ./environment-variables.sh

echo ""
echo "File: compile-funcs.sh"

export CONFIG_SYSTEM=$SYSTEM/../config-site

install_zlib() {
   print_info "Installing zlib"
   rm -rf "$ZLIB_SRC".obj
   mkdir -p "$ZLIB_SRC".obj
   cd "$ZLIB_SRC".obj
   CHOST=i686-pc-gnu \
   ../$ZLIB_SRC/configure \
     --prefix=$TARGET_ROOT
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$ZLIB_SRC".obj
}

install_flex() {
   print_info "Installing flex"
   cd "$FLEX_SRC"
   autoreconf -i
   cd ..
   rm -rf "$FLEX_SRC".obj
   mkdir -p "$FLEX_SRC".obj
   cd "$FLEX_SRC".obj
   CONFIG_SITE=$CONFIG_SYSTEM/flex-site.hurd-i386 \
     ../$FLEX_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT"
   sed -i -e 's/tests//' Makefile
   make -j$PROCS all
   fakeroot make install
   cd ..
   rm -rf "$FLEX_SRC".obj
}

install_mig() {
   print_info "Installing gnumig"
   cd "$GNUMIG_SRC"
   autoreconf -i
   cd ..
   rm -rf "$GNUMIG_SRC".obj
   mkdir -p "$GNUMIG_SRC".obj
   cd "$GNUMIG_SRC".obj
   rm -f config.cache
   CONFIG_SITE=$CONFIG_SYSTEM/mig-site.hurd-i386 \
   LDFLAGS="-L ${TARGET_ROOT}/lib" \
   ../$GNUMIG_SRC/configure \
      --build="$HOST" \
      --host="$TARGET" \
      --target="$TARGET" \
      --prefix="$TARGET_ROOT"
   make -j$PROCS all
   fakeroot make install
   cd ..
   rm -rf "$GNUMIG_SRC".obj
}

install_gnumach() {
   print_info "Installing gnumach"
   cd "$GNUMACH_SRC"
   autoreconf -i
   cd ..
   rm -rf "$GNUMACH_SRC".obj
   mkdir -p "$GNUMACH_SRC".obj
   cd "$GNUMACH_SRC".obj
   ../$GNUMACH_SRC/configure \
      --build="$HOST" \
      --host="$TARGET" \
      --enable-kdb \
      --enable-kmsg \
      --enable-pae \
      --prefix="$TARGET_ROOT"
   make -j$PROCS gnumach.gz gnumach gnumach.msgids
   fakeroot make install
   mkdir -p $TARGET_ROOT/boot
   cp gnumach.gz $TARGET_ROOT/boot
   cd -
   rm -rf "$GNUMACH_SRC".obj
}

install_pkg-config() {
   print_info "Installing pkg-config"
   cd "$PKG_CONFIG_SRC"
   autoreconf -i
   cd ..
   rm -rf "$PKG_CONFIG_SRC".obj
   mkdir -p "$PKG_CONFIG_SRC".obj
   cd "$PKG_CONFIG_SRC".obj
   CONFIG_SITE=$CONFIG_SYSTEM/pkg-config-site.hurd-i386 \
     ../"$PKG_CONFIG_SRC"/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --disable-shared \
     --with-internal-glib
   make -j$PROCS all-recursive
   fakeroot make install
   cd ..
   rm -rf "$PKG_CONFIG_SRC".obj
}

install_binutils ()
{
   print_info "Installing binutils"
   cd "$BINUTILS_SRC"
   autoreconf -i
   cd ..
   rm -rf "$BINUTILS_SRC".obj
   mkdir -p "$BINUTILS_SRC".obj
   cd "$BINUTILS_SRC".obj
   ../$BINUTILS_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --target="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --disable-nls \
     --enable-shared \
     --disable-multilib
   make -j$PROCS all
   fakeroot make install
   cd ..
   rm -rf "$BINUTILS_SRC".obj
}

install_sed() {
   print_info "Installing sed"
   cd "$SED_SRC"
   autoreconf -i
   cd ..
   rm -rf "$SED_SRC".obj
   mkdir -p "$SED_SRC".obj
   cd "$SED_SRC".obj
   ../$SED_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT"
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$SED_SRC".obj
}

install_coreutils() {
   print_info "Installing coreutils"
   cd "$COREUTILS_SRC"
   autoreconf -i
   cd ..
   rm -rf "$COREUTILS_SRC".obj
   mkdir -p "$COREUTILS_SRC".obj
   cd "$COREUTILS_SRC".obj
   CONFIG_SITE=$CONFIG_SYSTEM/coreutils-site.hurd-i386 \
   ../$COREUTILS_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --enable-install-program=hostname \
     --cache-file=config.cache
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$COREUTILS_SRC".obj
}

#FIXME: Needs gettext, gperf, texinfo: OK w/o
install_ncurses () {
  print_info "Installing ncurses"
# Don't: Requires autoconf-dickey
#  cd "$NCURSES_SRC"
#   autoreconf -i
#   cd ..
#
  rm -rf "$NCURSES_SRC".obj
  mkdir -p "$NCURSES_SRC".obj
  cd "$NCURSES_SRC".obj
  ../$NCURSES_SRC/configure \
    --build="$HOST" \
    --host="$TARGET" \
    --prefix="$TARGET_ROOT" \
    --includedir=$TARGET_ROOT/include/ncursesw \
    --with-shared \
    --without-debug \
    --without-ada \
    --with-pkg-config=$TARGET_ROOT/lib/pkgconfig \
    --with-pkg-config-libdir=yes \
    --enable-pc-files \
    --with-ticlib=tic \
    --with-termlib=tinfo \
    --disable-termcap \
    --enable-sysmlinks \
    --with-ncursesw \
    --enable-widec \
    --enable-overwrite \
    --with-build-cc=gcc
  make -j$PROCS
  fakeroot make install
  cd ..
  rm -rf "$NCURSES_SRC".obj
}

install_pam () {
  print_info "Installing pam"
  cd "$PAM_SRC"
  autoreconf -i
  cd ..
  rm -rf "$PAM_SRC".obj
  mkdir -p "$PAM_SRC".obj
  cd "$PAM_SRC".obj
  ../$PAM_SRC/configure \
    --build="$HOST" \
    --host="$TARGET" \
    --prefix="$TARGET_ROOT" \
    --includedir="$TARGET_ROOT"/include/security \
    --disable-cracklib \
    --enable-db=no
  make -j$PROCS
  fakeroot make install
  cd ..
  rm -rf "$PAM_SRC".obj
}

install_bash() {
   print_info "Installing bash"
   cd "$BASH_SRC"
   autoreconf -i
   cd ..
   rm -rf "$BASH_SRC".obj
   mkdir -p "$BASH_SRC".obj
   cd "$BASH_SRC".obj
   CONFIG_SITE=$CONFIG_SYSTEM/bash-site.hurd-i386 \
   ../$BASH_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --without-bash-malloc
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$BASH_SRC".obj
}

install_libgpg-error() {
   print_info "Installing libgpg-error"
   cd "$LIBGPG_ERROR_SRC"
   ./autogen.sh
   cd ..
   rm -rf "$LIBGPG_ERROR_SRC".obj
   mkdir -p "$LIBGPG_ERROR_SRC".obj
   cd "$LIBGPG_ERROR_SRC".obj
   ../$LIBGPG_ERROR_SRC/configure \
      --build="$HOST" \
      --host="$TARGET" \
      --prefix="$TARGET_ROOT"
   make -j$PROCS all
   fakeroot make install
   cd ..
   rm -rf "$LIBGPG_ERROR_SRC".obj
}

install_libgcrypt() {
   print_info "Installing libgcrypt"
   cd "$LIBGCRYPT_SRC"
   ./autogen.sh
   cd ..
   rm -rf "$LIBGCRYPT_SRC".obj
   mkdir -p "$LIBGCRYPT_SRC".obj
   cd "$LIBGCRYPT_SRC".obj
   CONFIG_SITE=$CONFIG_SYSTEM/libgcrypt-site.hurd-i386 \
   ../$LIBGCRYPT_SRC/configure \
      --build="$HOST" \
      --host="$TARGET" \
      --prefix="$TARGET_ROOT" \
      --with-libgpg-error-prefix="$TARGET_ROOT" \
      --enable-noexecstack \
      --enable-ld-version-script \
      --enable-static
      make -j$PROCS all
      fakeroot make install
   cd ..
   rm -rf "$LIBGCRYPT_SRC".obj
}

#FIXME: lwip uses cmake, not build!!
#FIXME: Download liblwip0 instead!!
install_lwip() {
   print_info "Installing lwip"
   cd "$LWIP_SRC"
   autoreconf -i
   cd ..
   mkdir -p "$LWIP_SRC".obj
   cd "$LWIP_SRC".obj
   rm -f config.cache config.status
   ../$LWIP_SRC/configure \
      --build="$HOST" \
      --host="$TARGET" \
      --prefix="$SYS_ROOT" \
      --without-parted \
      --disable-profile
   make -j$PROCS all
   fakeroot make install
   cd ..
   rm -rf "$LWIP_SRC".obj
}

install_libpciaccess() {
   print_info "Installing libpciaccess"
   cd "$LIBPCIACCESS_SRC"
   autoreconf -i
   cd ..
   rm -rf "$LIBPCIACCESS_SRC".obj
   mkdir -p "$LIBPCIACCESS_SRC".obj
   cd "$LIBPCIACCESS_SRC".obj
   ../"$LIBPCIACCESS_SRC"/configure \
      --build="$HOST" \
      --host="$TARGET" \
      --prefix="$TARGET_ROOT" \
      --enable-static
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$LIBPCIACCESS_SRC".obj
}

install_e2fsprogs() {
   print_info "Installing e2fsprogs"
   cd "$E2FSPROGS_SRC"
   autoreconf -i
   cd ..
   rm -rf "$E2FSPROGS_SRC".obj
   mkdir -p "$E2FSPROGS_SRC".obj
   cd "$E2FSPROGS_SRC".obj
   PKG_CONFIG_PATH=$TARGET_ROOT/lib/pkgconfig \
   ../$E2FSPROGS_SRC/configure \
      --build="$HOST" \
      --host="$TARGET" \
      --prefix="$TARGET_ROOT" \
      --enable-elf-shlibs \
      --enable-libuuid  \
      --disable-uuidd
   make -j$PROCS
   fakeroot make install
   fakeroot make install-libs
   cd ..
   rm -rf "$E2FSPROGS_SRC".obj
}

#FIXED: finish src_makefile.am.diff patch
#(cd tmp/src/shadow-4.8; patch -p1 < /home/srs/Guix/hurd-cross/patches/shadow/src_Makefile.am.diff)
#   apply_patch $SCRIPT_DIR/patches/shadow/src_Makefile.am.diff 1
#   cd ..
#   cd "$SHADOW_SRC"
#   unapply_patch $SCRIPT_DIR/patches/shadow/src_Makefile.am.diff 1
#   cd ..
install_shadow () {
   print_info "Installing shadow"
   cd "$SHADOW_SRC"
   autoreconf -i
   cd ..
   rm -rf "$SHADOW_SRC".obj
   mkdir "$SHADOW_SRC".obj
   cd "$SHADOW_SRC".obj
   CPPFLAGS="$CPPFLAGS -I$TARGET_ROOT/include" \
   LIBS="$LIBS -L$TARGET_ROOT/lib" \
   CONFIG_SITE=$CONFIG_SYSTEM/shadow-site.hurd-i386 \
   ../$SHADOW_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --cache-file=config.cache \
     --enable-shared \
     --with-libpam \
     --without-su \
     --enable-subordinate-ids
   make -j$PROCS all
   fakeroot make install
   cd ..
   rm -rf "$SHADOW_SRC".obj
}

# FIXME: Build also with libgcrypt -> /hurd/random,urandom !!
#   LDFLAGS="-Wl,-rpath-link,$SRC/$HURD_SRC.obj/lib -L$TARGET_ROOT/lib -lz" \
#
#      --with-libgcrypt-prefix="$TARGET_ROOT" \
#      --with-libcrypt \
#   CPPFLAGS="$CPPFLAGS -I$TARGET_ROOT/include" \
#   LDFLAGS="-Wl,-rpath-link,$SRC/$HURD_SRC.obj/lib" \
#
#      --without-libcrypt \
install_hurd() {
   print_info "Installing Hurd"
   cd "$HURD_SRC"
#   unapply_patch $SCRIPT_DIR/patches/hurd/doc_Makefile.diff 1
#   unapply_patch $SCRIPT_DIR/patches/hurd/trans_Makefile.diff 1
   apply_patch $SCRIPT_DIR/patches/hurd/doc_Makefile.diff 1
   apply_patch $SCRIPT_DIR/patches/hurd/trans_Makefile.diff 1
   autoreconf -i
   cd ..
   rm -rf "$HURD_SRC".obj
   mkdir -p "$HURD_SRC".obj
   cd "$HURD_SRC".obj
   PKG_CONFIG_PATH="$TARGET_ROOT"/lib/pkgconfig \
   libblkid_LIBS="-L$TARGET_ROOT/lib -lblkid -luuid" \
   LDFLAGS="-Wl,-rpath-link,$SRC/$HURD_SRC.obj/lib -L$TARGET_ROOT/lib -lz" \
   CPPFLAGS="$CPPFLAGS -I$TARGET_ROOT/include" \
   ../$HURD_SRC/configure \
      --build="$HOST" \
      --host="$TARGET" \
      --exec-prefix="$TARGET_ROOT" \
      --prefix="$TARGET_ROOT" \
      --without-parted \
      --disable-silent-rules \
      --without-libdaemon \
      --without-libbz2 \
      --enable-static-progs=ext2fs,iso9660fs \
      --with-libgcrypt-prefix="$TARGET_ROOT" \
      --with-libcrypt \
      --enable-ncursesw \
      --disable-profile
   make -j$PROCS all || exit 1
   fakeroot make install || exit 1
   cd ..
   cd "$HURD_SRC"
   unapply_patch $SCRIPT_DIR/patches/hurd/doc_Makefile.diff 1
   unapply_patch $SCRIPT_DIR/patches/hurd/trans_Makefile.diff 1
   cd ..
   rm -rf "$HURD_SRC".obj
}

install_util-linux() {
   print_info "Installing util-linux"
   cd "$UTIL_LINUX_SRC"
   ./autogen.sh
#   unapply_patch $SCRIPT_DIR/patches/util-linux/configure.diff 1
#   apply_patch $SCRIPT_DIR/patches/util-linux/configure.diff 1
   cd ..
   rm -rf "$UTIL_LINUX_SRC".obj
   mkdir -p "$UTIL_LINUX_SRC".obj
   cd "$UTIL_LINUX_SRC".obj
   PKG_CONFIG_PATH="$TARGET_ROOT/lib/pkgconfig" \
   CONFIG_SITE=$CONFIG_SYSTEM/util-linux-site.hurd-i386 \
   LDFLAGS="-Wl,-rpath-link,$SRC/$UTIL_LINUX_SRC.obj/.libs -L$TARGET_ROOT/lib" \
   CPPFLAGS="-I$TARGET_ROOT/include" \
   ../$UTIL_LINUX_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --with-bashcompletiondir=$TARGET_ROOT/share/bash-completion/completions \
     --with-ncursesw \
     --enable-widechar \
     --without-python \
     --without-systemd \
     --without-udev \
     --without-audit \
     --enable-libmount-support-mtab \
     --disable-hardlink \
     --disable-login \
     --disable-nologin \
     --disable-kill \
     --disable-eject \
     --disable-chfn-chsh \
     --disable-ul \
     --disable-cal
#   make V=1 -j$PROCS
   make -j$PROCS
   fakeroot make install
   cd ..
#   cd "$UTIL_LINUX_SRC"
#   unapply_patch $SCRIPT_DIR/patches/util-linux/configure.diff 1
#   cd ..
   rm -rf "$UTIL_LINUX_SRC".obj
}

install_grub() {
   print_info "Installing grub"
   cd "$GRUB_SRC"
   autoreconf -ivf
   cd ..
   rm -rf "$GRUB_SRC".obj
   mkdir -p "$GRUB_SRC".obj
   cd "$GRUB_SRC".obj
   ../$GRUB_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --disable-werror \
     --enable-grub-mkfont=no \
     --with-bootdir=$TARGET_ROOT/boot \
     --with-grubdir=$TARGET_ROOT/boot/grub
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$GRUB_SRC".obj
}

install_gmp() {
   print_info "Installing gmp"
   cd "$GMP_SRC"
   autoreconf -ivf
   cd ..
   rm -rf "$GMP_SRC".obj
   mkdir -p "$GMP_SRC".obj
   cd "$GMP_SRC".obj
   ../"$GMP_SRC"/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT"
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$GMP_SRC".obj
}

install_mpfr() {
   print_info "Installing mpfr"
   cd "$MPFR_SRC"
   autoreconf -i
   cd ..
   rm -rf "$MPFR_SRC".obj
   mkdir -p "$MPFR_SRC".obj
   cd "$MPFR_SRC".obj
   ../$MPFR_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --with-gmp-include="$TARGET_ROOT"/include \
     --with-gmp-lib="$TARGET_ROOT"/lib
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$MPFR_SRC".obj
}

install_mpc() {
   print_info "Installing mpc"
   cd "$MPC_SRC"
   autoreconf -i
   cd ..
   rm -rf "$MPC_SRC".obj
   mkdir -p "$MPC_SRC".obj
   cd "$MPC_SRC".obj
   ../"$MPC_SRC"/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --with-gmp-include="$TARGET_ROOT"/include \
     --with-gmp-lib="$TARGET_ROOT"/lib
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$MPC_SRC".obj
}

#(cd tmp/src/gcc-9.2.0; patch -p1 </home/srs/Guix/hurd-cross/patches/gcc/fixinc_glimits.h.diff)
#(cd tmp/src/gcc-9.2.0; patch -p1 </home/srs/Guix/hurd-cross/patches/gcc/mkfixinc.sh.diff)
install_gcc() {
   print_info "Installing gcc"
   cd "$GCC_SRC"
   autoreconf -ivf
   cd ..
   rm -rf "$GCC_SRC".obj
   mkdir -p "$GCC_SRC".obj
   cd "$GCC_SRC".obj
   LDFLAGS="-L$TARGET_ROOT/lib" \
   ../$GCC_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --target="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --with-gmp-include="$TARGET_ROOT"/include \
     --with-gmp-lib="$TARGET_ROOT"/lib \
     --enable-linker-build-id \
     --enable-lto \
     --enable-default-pie \
     --disable-multilib \
     --disable-nls \
     --disable-bootstrap \
     --enable-languages=c,c++ \
     --disable-libstdcxx-pch \
     --with-system-zlib \
     --disable-libcilkrts \
     --disable-libssp
   make -j$PROCS all
   fakeroot make install
   cd ..
#   rm -rf "$GCC_SRC".obj
}

install_glibc () {
   print_info "Installing glibc"
   cd "$GLIBC_SRC"
   autoreconf -i
   cd ..
   rm -rf "$GLIBC_SRC".obj
   mkdir -p "$GLIBC_SRC".obj
   cd "$GLIBC_SRC".obj
   rm -f config.cache
   CONFIG_SITE=$CONFIG_SYSTEM/glibc-site.hurd-i386 \
   ../"$GLIBC_SRC"/configure \
     --with-binutils=${ROOT}/bin \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --without-selinux \
     --enable-obsolete-rpc \
     --disable-profile \
     --disable-nscd \
     --enable-stackguard-randomization \
     --enable-stack-protector=strong \
     --enable-add-ons=libidn,"" \
     --enable-pt_chown \
     --disable-mathvec \
     --disable-werror
   make -j$PROCS
   fakeroot make install
   cd ..
#   rm -rf "$GLIBC_SRC".obj
}

install_nano() {
   print_info "Installing nano"
   cd "$NANO_SRC"
   autoreconf -i
   cd ..
   rm -rf "$NANO_SRC".obj
   mkdir -p "$NANO_SRC".obj
   cd "$NANO_SRC".obj
   PKG_CONFIG_PATH=$TARGET_ROOT/lib/pkgconfig \
   ../$NANO_SRC/configure \
     --build="$HOST" \
     --host="$TARGET" \
     --prefix="$TARGET_ROOT" \
     --enable-utf8=yes
   make -j$PROCS
   fakeroot make install
   cd ..
   rm -rf "$NANO_SRC".obj
}

install_vim () {
  print_info "Installing vim"
  cd vim"$VIM_BASE_VERSION"
  cat > src/auto/config.cache << "EOF"
  vim_cv_getcwd_broken=no
  vim_cv_memmove_handles_overlap=yes
  vim_cv_stat_ignores_slash=no
  vim_cv_terminfo=yes
  vim_cv_toupper_broken=no
  vim_cv_tty_group=world
EOF
  echo "#define SYS_VIMRC_FILE \"${SYS_ROOT}/etc/vimrc\"" >> src/feature.h
  ./configure --build=${HOST} \
     --host=${TARGET} \
    --prefix=${SYS_ROOT} \
    --enable-gui=no \
    --disable-gtktest \
    --disable-xim \
    --disable-gpm \
    --without-x \
    --disable-netbeans \
    --with-tlib=ncurses
  make
  make install
  ln -sfv vim $SYS_ROOT/bin/vi
  cd ..
  cat > $SYS_ROOT/etc/vimrc << "EOF"
set nocompatible
set backspace=2
set expandtab
set ts=2
set ruler
syntax on
EOF
}
