#!/bin/sh

# Copyright (C) 2019, 2020 Svante Signell

# Written by Svante Signell  <svante.signell@gmail.com>

# This file is part of hurd-cross.

# Hurd-cross is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 3 or (at your
# option) any later version.

# The hurd-cross is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with the GNU Hurd.  If not, see <http://www.gnu.org/licenses/>.


. ./compile-funcs.sh

echo ""
echo "File: compile.sh"

print_info "**************************************************"
print_info "Cross-compiling on $HOST to $TARGET"
print_info "**************************************************"

cd $SRC

if [ ! -f $STAMPS_COMPILE/$ZLIB_SRC ]; then
    install_zlib
    touch $STAMPS_COMPILE/$ZLIB_SRC
fi
#exit
if [ ! -f $STAMPS_COMPILE/$FLEX_SRC ]; then
    install_flex
    touch $STAMPS_COMPILE/$FLEX_SRC
fi
#exit
if [ ! -f $STAMPS_COMPILE/$GNUMIG_SRC-git ]; then
    install_mig
    touch $STAMPS_COMPILE/$GNUMIG_SRC-git
fi
#exit
if [ ! -f $STAMPS_COMPILE/$GNUMACH_SRC-git ]; then
    install_gnumach
    touch $STAMPS_COMPILE/$GNUMACH_SRC-git
fi
#exit
if [ ! -f $STAMPS_COMPILE/$PKG_CONFIG_SRC ]; then
    install_pkg-config || exit 1
    touch $STAMPS_COMPILE/$PKG_CONFIG_SRC
fi
#exit
if [ ! -f $STAMPS_COMPILE/$BINUTILS_SRC ]; then
    install_binutils
    touch $STAMPS_COMPILE/$BINUTILS_SRC
fi
#exit
if [ ! -f $STAMPS_COMPILE/$SED_SRC ]; then
    install_sed
    touch $STAMPS_COMPILE/$SED_SRC
fi
#exit
if [ ! -f $STAMPS_COMPILE/$COREUTILS_SRC ]; then
    install_coreutils
    touch $STAMPS_COMPILE/$COREUTILS_SRC
fi
#exit
if [ ! -f $STAMPS_COMPILE/$NCURSES_SRC ]; then
    install_ncurses
    touch $STAMPS_COMPILE/$NCURSES_SRC
fi
#exit
if [ ! -f $STAMPS_COMPILE/$PAM_SRC ]; then
    install_pam
    touch $STAMPS_COMPILE/$PAM_SRC
fi
#exit
if [ ! -f $STAMPS_COMPILE/$BASH_SRC ]; then
    install_bash
    touch $STAMPS_COMPILE/$BASH_SRC
fi
#exit
if [ ! -f $STAMPS_COMPILE/$LIBGPG_ERROR_SRC ]; then
    install_libgpg-error
    touch $STAMPS_COMPILE/$LIBGPG_ERROR_SRC
fi
#exit
if [ ! -f $STAMPS_COMPILE/$LIBGCRYPT_SRC ]; then
    install_libgcrypt
    touch $STAMPS_COMPILE/$LIBGCRYPT_SRC
fi
#exit

#FIXME: lwip needs cmake!!
#if [ ! -f lwip-stamp ]; then
#    install_lwip
#    touch lwip-stamp
#fi

# Needed by libpciaccess, util-linux and gcc: --disable-libtool-lock??
#if [ ! -f $COMPILE/$LIBTOOL_SRC ]; then
#    install_libtool
#    touch $COMPILE/$LIBTOOL_SRC
#fi

# Needed by netdde
if [ ! -f $STAMPS_COMPILE/$LIBPCIACCESS_SRC ]; then
    install_libpciaccess
    touch $STAMPS_COMPILE/$LIBPCIACCESS_SRC
fi
#exit
if [ ! -f $STAMPS_COMPILE/$E2FSPROGS_SRC ]; then
    install_e2fsprogs
    touch $STAMPS_COMPILE/$E2FSPROGS_SRC
fi
#exit
#FIXED: fails to build: add libuuid.so
if [ ! -f $STAMPS_COMPILE/$HURD_SRC-git ]; then
    install_hurd
    touch $STAMPS_COMPILE/$HURD_SRC-git
fi
#exit
#FIXME: Clean up; fix patches/shadow/src_Makefile.am.diff
if [ ! -f $STAMPS_COMPILE/$SHADOW_SRC ]; then
    install_shadow
    touch $STAMPS_COMPILE/$SHADOW_SRC
fi
#exit
#FIXME: Use a configure option to avoid patches/util-linux/configure.diff
if [ ! -f $STAMPS_COMPILE/$UTIL_LINUX_SRC ]; then
    install_util-linux || exit 1
    touch $STAMPS_COMPILE/$UTIL_LINUX_SRC
fi
#exit
if [ ! -f $STAMPS_COMPILE/$GRUB_SRC ]; then
    install_grub
    touch $STAMPS_COMPILE/$GRUB_SRC
fi
#exit
if [ ! -f $STAMPS_COMPILE/$NANO_SRC ]; then
    install_nano
    touch $STAMPS_COMPILE/$NANO_SRC
fi
#exit
if [ ! -f $STAMPS_COMPILE/$GMP_SRC ]; then
    install_gmp
    touch $STAMPS_COMPILE/$GMP_SRC
fi
#exit
if [ ! -f $STAMPS_COMPILE/$MPFR_SRC ]; then
    install_mpfr
    touch $STAMPS_COMPILE/$MPFR_SRC
fi
#exit
if [ ! -f $STAMPS_COMPILE/$MPC_SRC ]; then
    install_mpc
    touch $STAMPS_COMPILE/$MPC_SRC
fi
#exit
# Note: Build before glibc!! Clean up old installation of glibc before building!!
if [ ! -f $STAMPS_COMPILE/$GCC_SRC ]; then
    install_gcc
    touch $STAMPS_COMPILE/$GCC_SRC
fi
#exit  
# Note: Build after gcc!!
# Note: Flavios compile.sh does not build glibc, check the links !!!
if [ ! -f $STAMPS_COMPILE/$GLIBC_SRC ]; then
    install_glibc
    touch $STAMPS_COMPILE/$GLIBC_SRC
fi
#exit

#FIXED: Replace vim by nano, install vim later on.
#if [ ! -f vim-stamp ]; then
#    install_vim
#    touch vim-stamp
#fi

print_info "****************************************"
print_info "Cross-compiling and Installing completed"
print_info "****************************************"

exit 0
